@@ -104,8 +104,8 @@ static guint request_system_refcount = 0;
 static guint idle_stop_unused_threads_id = 0;
 static guint idle_close_unused_connections_id = 0;
 static GThreadPool *request_thread_pool;
-static GHashTable *request_map = 0;
-static GHashTable *connection_cache_map = 0;
+static GHashTable *request_map = NULL;
+static GHashTable *connection_cache_map = NULL;
 
 void
 _gnome_cups_request_init (GnomeCupsAuthFunction auth_fn)
@@ -280,7 +280,7 @@ dump_request (ipp_t const *req)
 		g_print ("%s", attr->name);
 		for (i = 0 ; i < attr->num_values ; i++) {
 			g_print ("\t[%d] = ", i);
-			switch (attr->value_tag) {
+			switch (attr->value_tag & ~IPP_TAG_COPY) {
 			case IPP_TAG_INTEGER:
 			case IPP_TAG_ENUM:
 				g_print ("%d\n", attr->values[i].integer);
@@ -298,7 +298,7 @@ dump_request (ipp_t const *req)
 				break;
 
 			case IPP_TAG_BOOLEAN:
-				g_print ("%d\n", (int)attr->values[i].boolean);
+				g_print ("%s\n", (int)attr->values[i].boolean ? "true" : "false");
 				break;
 
 			default:
@@ -396,23 +396,17 @@ _gnome_cups_outstanding_request_count (void)
 ipp_t *
 gnome_cups_request_new (int operation_id)
 {
-	ipp_t *request;
 	cups_lang_t *language;
+	ipp_t *request;
+	
+	request = ippNewRequest (operation_id);
 	
 	language = cupsLangDefault ();
-	request = ippNew ();
-	request->request.op.operation_id = operation_id;
-	request->request.op.request_id = 1;
-	
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
-		     "attributes-charset", 
-		     NULL, "utf-8");
-	
-	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
-		     "attributes-natural-language", 
-		     NULL, language->language);
+	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+		      "attributes-natural-language", 
+		      NULL, language->language);
 	cupsLangFree (language);
-
+	
 	return request;
 }
 
@@ -469,18 +463,10 @@ gnome_cups_request_add_requested_attributes (ipp_t *request,
 					     int n_attributes,
 					     char **attributes)
 {
-	ipp_attribute_t *attr;
-	int i;
-	
-	attr = ippAddStrings (request, 
-			      group,
-			      IPP_TAG_KEYWORD,
-			      "requested-attributes",
-			      n_attributes, NULL, NULL);
-
-	for (i = 0; i < n_attributes; i++) {
-		attr->values[i].string.text = gnome_cups_strdup (attributes[i]);
-	}
+	ippAddStrings (request, group,
+		       IPP_TAG_NAME | IPP_TAG_COPY,
+		       "requested-attributes",
+		       n_attributes, NULL, attributes);
 }
 
 typedef struct
@@ -631,6 +617,7 @@ gnome_cups_request_execute_async_internal (ipp_t      *request,
 {
 	GnomeCupsConnection *connection;
 	GnomeCupsRequest *req;
+	gint retval;
 
 	if (!server)
 		server = cupsServer();
@@ -671,9 +658,10 @@ gnome_cups_request_execute_async_internal (ipp_t      *request,
 	g_thread_pool_push (request_thread_pool, req, NULL);
 
 	g_hash_table_insert (request_map, GUINT_TO_POINTER (req->id), req);
+	retval=req->id;
 	g_static_mutex_unlock (&request_mutex);
 
-	return req->id;
+	return retval;
 }
 
 void
